// Use subject to GPLv3.  Copyright 2011 George Magiros
#include <avr/io.h>
#include <avr/wdt.h>         // for wdt routines
#include <avr/interrupt.h>   // for sei() 
#include <util/delay.h>      // for _delay_ms()
#include <string.h>

#include <avr/pgmspace.h>    // required by usbdrv.h
#include "usbdrv.h"
#include "oddebug.h"         // debug macros


// yat ser/hid compatible implementation. input 16 bytes / output 16 bytes

/* USB report descriptor */
PROGMEM const char usbHidReportDescriptor[] = {
  0x06, 0xa0, 0xff, // USAGE_PAGE (Vendor Defined Page 1)
  0x09, 0x01,       // USAGE (Vendor Usage 1)
  0xa1, 0x01,       // COLLECTION (Application)

  // Input Report
  0x09, 0x02,       // Usage ID - vendor defined
  0x15, 0x00,       // Logical Minimum (0)
  0x26, 0xFF, 0x00, // Logical Maximum (255)
  0x75, 0x08,       // Report Size (8 bits)
  0x95, 0x08,       // Report Count (8 fields)	// must be 8
  0x81, 0x02,       // Input (Data, Variable, Absolute)

  // Output report
  0x09, 0x03,       // Usage ID - vendor defined
  0x15, 0x00,       // Logical Minimum (0)
  0x26, 0xFF, 0x00, // Logical Maximum (255)
  0x75, 0x08,       // Report Size (8 bits)
  0x95, 0x16,       // Report Count (8 fields)	// change for buffer setup
  0x91, 0x02,       // Output (Data, Variable, Absolute)

  0xc0              // END_COLLECTION
};

static uchar inbuf[16], outbuf[16], outlen=0;

void usbFunctionWriteOut (uchar *data, uchar len)
{
  if(len>0) {	// output(host -> usb hid) can be NULL
  	if((data[0]!=0x0d)&&(data[0]!=0x0a)) {
  		uchar i;
  		if(len>16) len=16;
  		for (i=0; i<len; i++) outbuf[i] = data[i];
  		outlen = len;
  	}
  }
}


usbMsgLen_t usbFunctionSetup (uchar *data)
{
//usbRequest_t *rq = (usbRequest_t *)data;
//	if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){ /* HID class request */
//	if(rq->bRequest == USBRQ_HID_GET_REPORT){
//		return USB_NO_MSG; /* use usbFunctionRead() to obtain data */
//	}else if(rq->bRequest == USBRQ_HID_SET_REPORT){
//		return USB_NO_MSG; /* use usbFunctionWrite() to receive data from host */
//	}
//	}else{
//	/* ignore vendor type requests, we don't use any */
//	}
	return 0;
}


void chartohex(char c,char *dest)
{
uchar n1=c >> 4;
uchar n2=c & 0x0f;
	if(n1>9) *dest='A'+(n1-10);
		else *dest='0'+n1;
	dest++;
	if(n2>9) *dest='A'+(n2-10);
		else *dest='0'+n2;
}


void main_loop (void)
{
  uchar i,j,c;
  if (outlen) {
    if (usbInterruptIsReady()) {
	  i=0; j=0;
	  while(i<outlen) {
	  	c=outbuf[i];
		if(c==0) break;
		chartohex(c,(char*)(&inbuf[j]));
		j+=2;
	  	i++;
		if(j>=15) break;
	  }
	  inbuf[j]=0;
	  // result 16 bytes, for more bytes add usbPoll() & usbSetInterrupt()
	  usbSetInterrupt(inbuf,8);
	  if(j>8) {
	  	while(!usbInterruptIsReady()) usbPoll();
	  	usbSetInterrupt(inbuf+8,8);
	  }
      outlen = 0;
    }
  }
}


int main (void)
{
  wdt_disable();
  odDebugInit();
  usbInit();

  usbDeviceDisconnect();  // force re-enumeration
  _delay_ms(250);
  usbDeviceConnect();

  sei();                  // now enable interrupts
  for(;;){
    usbPoll();
    main_loop();
  }
}

